@@ -80,67 +80,68 @@
      BX_TOKEN_LDT = 296,
      BX_TOKEN_TSS = 297,
      BX_TOKEN_TAB = 298,
-     BX_TOKEN_DIRTY = 299,
+     BX_TOKEN_ALL = 299,
      BX_TOKEN_LINUX = 300,
-     BX_TOKEN_CONTROL_REGS = 301,
-     BX_TOKEN_SEGMENT_REGS = 302,
-     BX_TOKEN_EXAMINE = 303,
-     BX_TOKEN_XFORMAT = 304,
-     BX_TOKEN_DISFORMAT = 305,
-     BX_TOKEN_RESTORE = 306,
-     BX_TOKEN_SETPMEM = 307,
-     BX_TOKEN_SYMBOLNAME = 308,
-     BX_TOKEN_QUERY = 309,
-     BX_TOKEN_PENDING = 310,
-     BX_TOKEN_TAKE = 311,
-     BX_TOKEN_DMA = 312,
-     BX_TOKEN_IRQ = 313,
-     BX_TOKEN_DISASSEMBLE = 314,
-     BX_TOKEN_INSTRUMENT = 315,
-     BX_TOKEN_STRING = 316,
-     BX_TOKEN_STOP = 317,
-     BX_TOKEN_DOIT = 318,
-     BX_TOKEN_CRC = 319,
-     BX_TOKEN_TRACE = 320,
-     BX_TOKEN_TRACEREG = 321,
-     BX_TOKEN_TRACEMEM = 322,
-     BX_TOKEN_SWITCH_MODE = 323,
-     BX_TOKEN_SIZE = 324,
-     BX_TOKEN_PTIME = 325,
-     BX_TOKEN_TIMEBP_ABSOLUTE = 326,
-     BX_TOKEN_TIMEBP = 327,
-     BX_TOKEN_RECORD = 328,
-     BX_TOKEN_PLAYBACK = 329,
-     BX_TOKEN_MODEBP = 330,
-     BX_TOKEN_PRINT_STACK = 331,
-     BX_TOKEN_WATCH = 332,
-     BX_TOKEN_UNWATCH = 333,
-     BX_TOKEN_READ = 334,
-     BX_TOKEN_WRITE = 335,
-     BX_TOKEN_SHOW = 336,
-     BX_TOKEN_LOAD_SYMBOLS = 337,
-     BX_TOKEN_SYMBOLS = 338,
-     BX_TOKEN_LIST_SYMBOLS = 339,
-     BX_TOKEN_GLOBAL = 340,
-     BX_TOKEN_WHERE = 341,
-     BX_TOKEN_PRINT_STRING = 342,
-     BX_TOKEN_NUMERIC = 343,
-     BX_TOKEN_NE2000 = 344,
-     BX_TOKEN_PIC = 345,
-     BX_TOKEN_PAGE = 346,
-     BX_TOKEN_HELP = 347,
-     BX_TOKEN_CALC = 348,
-     BX_TOKEN_VGA = 349,
-     BX_TOKEN_PCI = 350,
-     BX_TOKEN_COMMAND = 351,
-     BX_TOKEN_GENERIC = 352,
-     BX_TOKEN_RSHIFT = 353,
-     BX_TOKEN_LSHIFT = 354,
-     BX_TOKEN_REG_IP = 355,
-     BX_TOKEN_REG_EIP = 356,
-     BX_TOKEN_REG_RIP = 357,
-     NEG = 358,
-     NOT = 359
+     BX_TOKEN_DEBUG_REGS = 301,
+     BX_TOKEN_CONTROL_REGS = 302,
+     BX_TOKEN_SEGMENT_REGS = 303,
+     BX_TOKEN_EXAMINE = 304,
+     BX_TOKEN_XFORMAT = 305,
+     BX_TOKEN_DISFORMAT = 306,
+     BX_TOKEN_RESTORE = 307,
+     BX_TOKEN_SETPMEM = 308,
+     BX_TOKEN_SYMBOLNAME = 309,
+     BX_TOKEN_QUERY = 310,
+     BX_TOKEN_PENDING = 311,
+     BX_TOKEN_TAKE = 312,
+     BX_TOKEN_DMA = 313,
+     BX_TOKEN_IRQ = 314,
+     BX_TOKEN_DISASSEMBLE = 315,
+     BX_TOKEN_INSTRUMENT = 316,
+     BX_TOKEN_STRING = 317,
+     BX_TOKEN_STOP = 318,
+     BX_TOKEN_DOIT = 319,
+     BX_TOKEN_CRC = 320,
+     BX_TOKEN_TRACE = 321,
+     BX_TOKEN_TRACEREG = 322,
+     BX_TOKEN_TRACEMEM = 323,
+     BX_TOKEN_SWITCH_MODE = 324,
+     BX_TOKEN_SIZE = 325,
+     BX_TOKEN_PTIME = 326,
+     BX_TOKEN_TIMEBP_ABSOLUTE = 327,
+     BX_TOKEN_TIMEBP = 328,
+     BX_TOKEN_RECORD = 329,
+     BX_TOKEN_PLAYBACK = 330,
+     BX_TOKEN_MODEBP = 331,
+     BX_TOKEN_PRINT_STACK = 332,
+     BX_TOKEN_WATCH = 333,
+     BX_TOKEN_UNWATCH = 334,
+     BX_TOKEN_READ = 335,
+     BX_TOKEN_WRITE = 336,
+     BX_TOKEN_SHOW = 337,
+     BX_TOKEN_LOAD_SYMBOLS = 338,
+     BX_TOKEN_SYMBOLS = 339,
+     BX_TOKEN_LIST_SYMBOLS = 340,
+     BX_TOKEN_GLOBAL = 341,
+     BX_TOKEN_WHERE = 342,
+     BX_TOKEN_PRINT_STRING = 343,
+     BX_TOKEN_NUMERIC = 344,
+     BX_TOKEN_NE2000 = 345,
+     BX_TOKEN_PIC = 346,
+     BX_TOKEN_PAGE = 347,
+     BX_TOKEN_HELP = 348,
+     BX_TOKEN_CALC = 349,
+     BX_TOKEN_VGA = 350,
+     BX_TOKEN_PCI = 351,
+     BX_TOKEN_COMMAND = 352,
+     BX_TOKEN_GENERIC = 353,
+     BX_TOKEN_RSHIFT = 354,
+     BX_TOKEN_LSHIFT = 355,
+     BX_TOKEN_REG_IP = 356,
+     BX_TOKEN_REG_EIP = 357,
+     BX_TOKEN_REG_RIP = 358,
+     NEG = 359,
+     NOT = 360
    };
 #endif
 /* Tokens.  */
@@ -185,67 +186,68 @@
 #define BX_TOKEN_LDT 296
 #define BX_TOKEN_TSS 297
 #define BX_TOKEN_TAB 298
-#define BX_TOKEN_DIRTY 299
+#define BX_TOKEN_ALL 299
 #define BX_TOKEN_LINUX 300
-#define BX_TOKEN_CONTROL_REGS 301
-#define BX_TOKEN_SEGMENT_REGS 302
-#define BX_TOKEN_EXAMINE 303
-#define BX_TOKEN_XFORMAT 304
-#define BX_TOKEN_DISFORMAT 305
-#define BX_TOKEN_RESTORE 306
-#define BX_TOKEN_SETPMEM 307
-#define BX_TOKEN_SYMBOLNAME 308
-#define BX_TOKEN_QUERY 309
-#define BX_TOKEN_PENDING 310
-#define BX_TOKEN_TAKE 311
-#define BX_TOKEN_DMA 312
-#define BX_TOKEN_IRQ 313
-#define BX_TOKEN_DISASSEMBLE 314
-#define BX_TOKEN_INSTRUMENT 315
-#define BX_TOKEN_STRING 316
-#define BX_TOKEN_STOP 317
-#define BX_TOKEN_DOIT 318
-#define BX_TOKEN_CRC 319
-#define BX_TOKEN_TRACE 320
-#define BX_TOKEN_TRACEREG 321
-#define BX_TOKEN_TRACEMEM 322
-#define BX_TOKEN_SWITCH_MODE 323
-#define BX_TOKEN_SIZE 324
-#define BX_TOKEN_PTIME 325
-#define BX_TOKEN_TIMEBP_ABSOLUTE 326
-#define BX_TOKEN_TIMEBP 327
-#define BX_TOKEN_RECORD 328
-#define BX_TOKEN_PLAYBACK 329
-#define BX_TOKEN_MODEBP 330
-#define BX_TOKEN_PRINT_STACK 331
-#define BX_TOKEN_WATCH 332
-#define BX_TOKEN_UNWATCH 333
-#define BX_TOKEN_READ 334
-#define BX_TOKEN_WRITE 335
-#define BX_TOKEN_SHOW 336
-#define BX_TOKEN_LOAD_SYMBOLS 337
-#define BX_TOKEN_SYMBOLS 338
-#define BX_TOKEN_LIST_SYMBOLS 339
-#define BX_TOKEN_GLOBAL 340
-#define BX_TOKEN_WHERE 341
-#define BX_TOKEN_PRINT_STRING 342
-#define BX_TOKEN_NUMERIC 343
-#define BX_TOKEN_NE2000 344
-#define BX_TOKEN_PIC 345
-#define BX_TOKEN_PAGE 346
-#define BX_TOKEN_HELP 347
-#define BX_TOKEN_CALC 348
-#define BX_TOKEN_VGA 349
-#define BX_TOKEN_PCI 350
-#define BX_TOKEN_COMMAND 351
-#define BX_TOKEN_GENERIC 352
-#define BX_TOKEN_RSHIFT 353
-#define BX_TOKEN_LSHIFT 354
-#define BX_TOKEN_REG_IP 355
-#define BX_TOKEN_REG_EIP 356
-#define BX_TOKEN_REG_RIP 357
-#define NEG 358
-#define NOT 359
+#define BX_TOKEN_DEBUG_REGS 301
+#define BX_TOKEN_CONTROL_REGS 302
+#define BX_TOKEN_SEGMENT_REGS 303
+#define BX_TOKEN_EXAMINE 304
+#define BX_TOKEN_XFORMAT 305
+#define BX_TOKEN_DISFORMAT 306
+#define BX_TOKEN_RESTORE 307
+#define BX_TOKEN_SETPMEM 308
+#define BX_TOKEN_SYMBOLNAME 309
+#define BX_TOKEN_QUERY 310
+#define BX_TOKEN_PENDING 311
+#define BX_TOKEN_TAKE 312
+#define BX_TOKEN_DMA 313
+#define BX_TOKEN_IRQ 314
+#define BX_TOKEN_DISASSEMBLE 315
+#define BX_TOKEN_INSTRUMENT 316
+#define BX_TOKEN_STRING 317
+#define BX_TOKEN_STOP 318
+#define BX_TOKEN_DOIT 319
+#define BX_TOKEN_CRC 320
+#define BX_TOKEN_TRACE 321
+#define BX_TOKEN_TRACEREG 322
+#define BX_TOKEN_TRACEMEM 323
+#define BX_TOKEN_SWITCH_MODE 324
+#define BX_TOKEN_SIZE 325
+#define BX_TOKEN_PTIME 326
+#define BX_TOKEN_TIMEBP_ABSOLUTE 327
+#define BX_TOKEN_TIMEBP 328
+#define BX_TOKEN_RECORD 329
+#define BX_TOKEN_PLAYBACK 330
+#define BX_TOKEN_MODEBP 331
+#define BX_TOKEN_PRINT_STACK 332
+#define BX_TOKEN_WATCH 333
+#define BX_TOKEN_UNWATCH 334
+#define BX_TOKEN_READ 335
+#define BX_TOKEN_WRITE 336
+#define BX_TOKEN_SHOW 337
+#define BX_TOKEN_LOAD_SYMBOLS 338
+#define BX_TOKEN_SYMBOLS 339
+#define BX_TOKEN_LIST_SYMBOLS 340
+#define BX_TOKEN_GLOBAL 341
+#define BX_TOKEN_WHERE 342
+#define BX_TOKEN_PRINT_STRING 343
+#define BX_TOKEN_NUMERIC 344
+#define BX_TOKEN_NE2000 345
+#define BX_TOKEN_PIC 346
+#define BX_TOKEN_PAGE 347
+#define BX_TOKEN_HELP 348
+#define BX_TOKEN_CALC 349
+#define BX_TOKEN_VGA 350
+#define BX_TOKEN_PCI 351
+#define BX_TOKEN_COMMAND 352
+#define BX_TOKEN_GENERIC 353
+#define BX_TOKEN_RSHIFT 354
+#define BX_TOKEN_LSHIFT 355
+#define BX_TOKEN_REG_IP 356
+#define BX_TOKEN_REG_EIP 357
+#define BX_TOKEN_REG_RIP 358
+#define NEG 359
+#define NOT 360
 
 
 
@@ -259,7 +261,7 @@ typedef union YYSTYPE
   bx_bool  bval;
 }
 /* Line 1489 of yacc.c.  */
-#line 263 "y.tab.h"
+#line 265 "y.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
