@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: access64.cc,v 1.24 2009/03/13 18:48:08 sshwarts Exp $
+// $Id: access64.cc,v 1.26 2009/10/14 20:45:29 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2008 Stanislav Shwartsman
+//   Copyright (c) 2008-2009 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -94,7 +94,7 @@ BX_CPU_C::write_virtual_word_64(unsigned s, Bit64u offset, Bit16u data)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+1)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_virtual_word_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -108,6 +108,11 @@ BX_CPU_C::write_virtual_word_64(unsigned s, Bit64u offset, Bit16u data)
   }
 #endif
 
+  if (! IsCanonical(laddr+1)) {
+    BX_ERROR(("write_virtual_word_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_write_linear(laddr, 2, CPL, (void *) &data);
 }
 
@@ -142,7 +147,7 @@ BX_CPU_C::write_virtual_dword_64(unsigned s, Bit64u offset, Bit32u data)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+3)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_virtual_dword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -156,6 +161,11 @@ BX_CPU_C::write_virtual_dword_64(unsigned s, Bit64u offset, Bit32u data)
   }
 #endif
 
+  if (! IsCanonical(laddr+3)) {
+    BX_ERROR(("write_virtual_dword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_write_linear(laddr, 4, CPL, (void *) &data);
 }
 
@@ -190,7 +200,7 @@ BX_CPU_C::write_virtual_qword_64(unsigned s, Bit64u offset, Bit64u data)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+7)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_virtual_qword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -204,6 +214,11 @@ BX_CPU_C::write_virtual_qword_64(unsigned s, Bit64u offset, Bit64u data)
   }
 #endif
 
+  if (! IsCanonical(laddr+7)) {
+    BX_ERROR(("write_virtual_qword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_write_linear(laddr, 8, CPL, (void *) &data);
 }
 
@@ -349,7 +364,7 @@ BX_CPU_C::read_virtual_word_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+1)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_virtual_word_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -363,6 +378,11 @@ BX_CPU_C::read_virtual_word_64(unsigned s, Bit64u offset)
   }
 #endif
 
+  if (! IsCanonical(laddr+1)) {
+    BX_ERROR(("read_virtual_word_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 2, CPL, BX_READ, (void *) &data);
   return data;
 }
@@ -397,7 +417,7 @@ BX_CPU_C::read_virtual_dword_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+3)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_virtual_dword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -411,6 +431,11 @@ BX_CPU_C::read_virtual_dword_64(unsigned s, Bit64u offset)
   }
 #endif
 
+  if (! IsCanonical(laddr+3)) {
+    BX_ERROR(("read_virtual_dword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 4, CPL, BX_READ, (void *) &data);
   return data;
 }
@@ -445,7 +470,7 @@ BX_CPU_C::read_virtual_qword_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+7)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_virtual_qword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -458,6 +483,12 @@ BX_CPU_C::read_virtual_qword_64(unsigned s, Bit64u offset)
     }
   }
 #endif
+
+  if (! IsCanonical(laddr+7)) {
+    BX_ERROR(("read_virtual_qword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 8, CPL, BX_READ, (void *) &data);
   return data;
 }
@@ -609,7 +640,7 @@ BX_CPU_C::read_RMW_virtual_word_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+1)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_RMW_virtual_word_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -623,6 +654,11 @@ BX_CPU_C::read_RMW_virtual_word_64(unsigned s, Bit64u offset)
   }
 #endif
 
+  if (! IsCanonical(laddr+1)) {
+    BX_ERROR(("read_RMW_virtual_word_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 2, CPL, BX_RW, (void *) &data);
   return data;
 }
@@ -659,7 +695,7 @@ BX_CPU_C::read_RMW_virtual_dword_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+3)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_RMW_virtual_dword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -673,6 +709,11 @@ BX_CPU_C::read_RMW_virtual_dword_64(unsigned s, Bit64u offset)
   }
 #endif
 
+  if (! IsCanonical(laddr+3)) {
+    BX_ERROR(("read_RMW_virtual_dword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 4, CPL, BX_RW, (void *) &data);
   return data;
 }
@@ -709,7 +750,7 @@ BX_CPU_C::read_RMW_virtual_qword_64(unsigned s, Bit64u offset)
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+7)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("read_RMW_virtual_qword_64(): canonical failure"));
     exception(int_number(s), 0, 0);
   }
@@ -723,6 +764,11 @@ BX_CPU_C::read_RMW_virtual_qword_64(unsigned s, Bit64u offset)
   }
 #endif
 
+  if (! IsCanonical(laddr+7)) {
+    BX_ERROR(("read_RMW_virtual_qword_64(): canonical failure"));
+    exception(int_number(s), 0, 0);
+  }
+
   access_read_linear(laddr, 8, CPL, BX_RW, (void *) &data);
   return data;
 }
@@ -753,7 +799,7 @@ void BX_CPU_C::write_new_stack_word_64(Bit64u laddr, unsigned curr_pl, Bit16u da
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+1)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_new_stack_word_64(): canonical failure"));
     exception(BX_SS_EXCEPTION, 0, 0);
   }
@@ -767,6 +813,11 @@ void BX_CPU_C::write_new_stack_word_64(Bit64u laddr, unsigned curr_pl, Bit16u da
   }
 #endif
 
+  if (! IsCanonical(laddr+1)) {
+    BX_ERROR(("write_new_stack_word_64(): canonical failure"));
+    exception(BX_SS_EXCEPTION, 0, 0);
+  }
+
   access_write_linear(laddr, 2, curr_pl, (void *) &data);
 }
 
@@ -796,7 +847,7 @@ void BX_CPU_C::write_new_stack_dword_64(Bit64u laddr, unsigned curr_pl, Bit32u d
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+3)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_new_stack_dword_64(): canonical failure"));
     exception(BX_SS_EXCEPTION, 0, 0);
   }
@@ -810,6 +861,11 @@ void BX_CPU_C::write_new_stack_dword_64(Bit64u laddr, unsigned curr_pl, Bit32u d
   }
 #endif
 
+  if (! IsCanonical(laddr+3)) {
+    BX_ERROR(("write_new_stack_dword_64(): canonical failure"));
+    exception(BX_SS_EXCEPTION, 0, 0);
+  }
+
   access_write_linear(laddr, 4, curr_pl, (void *) &data);
 }
 
@@ -839,7 +895,7 @@ void BX_CPU_C::write_new_stack_qword_64(Bit64u laddr, unsigned curr_pl, Bit64u d
     }
   }
 
-  if (! IsCanonical(laddr) || ! IsCanonical(laddr+7)) {
+  if (! IsCanonical(laddr)) {
     BX_ERROR(("write_new_stack_qword_64(): canonical failure"));
     exception(BX_SS_EXCEPTION, 0, 0);
   }
@@ -853,6 +909,11 @@ void BX_CPU_C::write_new_stack_qword_64(Bit64u laddr, unsigned curr_pl, Bit64u d
   }
 #endif
 
+  if (! IsCanonical(laddr+7)) {
+    BX_ERROR(("write_new_stack_qword_64(): canonical failure"));
+    exception(BX_SS_EXCEPTION, 0, 0);
+  }
+
   access_write_linear(laddr, 8, curr_pl, (void *) &data);
 }
 
