@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith16.cc,v 1.61 2007/12/23 17:21:27 sshwarts Exp $
+// $Id: arith16.cc,v 1.72 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,33 +22,33 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
-void BX_CPU_C::INC_RX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_RX(bxInstruction_c *i)
 {
   Bit16u rx = ++BX_READ_16BIT_REG(i->opcodeReg());
   SET_FLAGS_OSZAPC_INC_16(rx);
 }
 
-void BX_CPU_C::DEC_RX(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_RX(bxInstruction_c *i)
 {
   Bit16u rx = --BX_READ_16BIT_REG(i->opcodeReg());
   SET_FLAGS_OSZAPC_DEC_16(rx);
 }
 
-void BX_CPU_C::ADD_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   sum_16 = op1_16 + op2_16;
   write_RMW_virtual_word(sum_16);
@@ -56,32 +56,7 @@ void BX_CPU_C::ADD_EwGwM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 }
 
-void BX_CPU_C::ADD_EwGwR(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, sum_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->rm());
-  op2_16 = BX_READ_16BIT_REG(i->nnn());
-  sum_16 = op1_16 + op2_16;
-  BX_WRITE_16BIT_REG(i->rm(), sum_16);
-
-  SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
-}
-
-void BX_CPU_C::ADD_GwEwM(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, sum_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-  op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  sum_16 = op1_16 + op2_16;
-
-  BX_WRITE_16BIT_REG(i->nnn(), sum_16);
-
-  SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
-}
-
-void BX_CPU_C::ADD_GwEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
 
@@ -94,7 +69,7 @@ void BX_CPU_C::ADD_GwEwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 }
 
-void BX_CPU_C::ADD_AXIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
 
@@ -106,46 +81,22 @@ void BX_CPU_C::ADD_AXIw(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 }
 
-void BX_CPU_C::ADC_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
   bx_bool temp_CF = getB_CF();
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   sum_16 = op1_16 + op2_16 + temp_CF;
   write_RMW_virtual_word(sum_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_LF_INSTR_ADD_ADC16(temp_CF));
 }
 
-void BX_CPU_C::ADC_EwGwR(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, sum_16;
-  bx_bool temp_CF = getB_CF();
-
-  op1_16 = BX_READ_16BIT_REG(i->rm());
-  op2_16 = BX_READ_16BIT_REG(i->nnn());
-  sum_16 = op1_16 + op2_16 + temp_CF;
-  BX_WRITE_16BIT_REG(i->rm(), sum_16);
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
-}
-
-void BX_CPU_C::ADC_GwEwM(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, sum_16;
-  bx_bool temp_CF = getB_CF();
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-  op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  sum_16 = op1_16 + op2_16 + temp_CF;
-  BX_WRITE_16BIT_REG(i->nnn(), sum_16);
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
-}
-
-void BX_CPU_C::ADC_GwEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
   bx_bool temp_CF = getB_CF();
@@ -155,10 +106,10 @@ void BX_CPU_C::ADC_GwEwR(bxInstruction_c *i)
   sum_16 = op1_16 + op2_16 + temp_CF;
   BX_WRITE_16BIT_REG(i->nnn(), sum_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_LF_INSTR_ADD_ADC16(temp_CF));
 }
 
-void BX_CPU_C::ADC_AXIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, sum_16;
   bx_bool temp_CF = getB_CF();
@@ -168,49 +119,25 @@ void BX_CPU_C::ADC_AXIw(bxInstruction_c *i)
   sum_16 = op1_16 + op2_16 + temp_CF;
   AX = sum_16;
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_LF_INSTR_ADD_ADC16(temp_CF));
 }
 
-void BX_CPU_C::SBB_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
   bx_bool temp_CF = getB_CF();
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   diff_16 = op1_16 - (op2_16 + temp_CF);
   write_RMW_virtual_word(diff_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_LF_INSTR_SUB_SBB16(temp_CF));
 }
 
-void BX_CPU_C::SBB_EwGwR(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-  bx_bool temp_CF = getB_CF();
-
-  op1_16 = BX_READ_16BIT_REG(i->rm());
-  op2_16 = BX_READ_16BIT_REG(i->nnn());
-  diff_16 = op1_16 - (op2_16 + temp_CF);
-  BX_WRITE_16BIT_REG(i->rm(), diff_16);
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
-}
-
-void BX_CPU_C::SBB_GwEwM(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-  bx_bool temp_CF = getB_CF();
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-  op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  diff_16 = op1_16 - (op2_16 + temp_CF);
-  BX_WRITE_16BIT_REG(i->nnn(), diff_16);
-
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
-}
-
-void BX_CPU_C::SBB_GwEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
   bx_bool temp_CF = getB_CF();
@@ -220,10 +147,10 @@ void BX_CPU_C::SBB_GwEwR(bxInstruction_c *i)
   diff_16 = op1_16 - (op2_16 + temp_CF);
   BX_WRITE_16BIT_REG(i->nnn(), diff_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_LF_INSTR_SUB_SBB16(temp_CF));
 }
 
-void BX_CPU_C::SBB_AXIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_AXIw(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
   Bit16u op1_16, op2_16, diff_16;
@@ -233,22 +160,24 @@ void BX_CPU_C::SBB_AXIw(bxInstruction_c *i)
   diff_16 = op1_16 - (op2_16 + temp_CF);
   AX = diff_16;
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_LF_INSTR_SUB_SBB16(temp_CF));
 }
 
-void BX_CPU_C::SBB_EwIwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EwIwM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   diff_16 = op1_16 - (op2_16 + temp_CF);
   write_RMW_virtual_word(diff_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_LF_INSTR_SUB_SBB16(temp_CF));
 }
 
-void BX_CPU_C::SBB_EwIwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EwIwR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
@@ -257,14 +186,16 @@ void BX_CPU_C::SBB_EwIwR(bxInstruction_c *i)
   diff_16 = op1_16 - (op2_16 + temp_CF);
   BX_WRITE_16BIT_REG(i->rm(), diff_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_INSTR_SUB_SBB16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, diff_16, BX_LF_INSTR_SUB_SBB16(temp_CF));
 }
 
-void BX_CPU_C::SUB_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   diff_16 = op1_16 - op2_16;
   write_RMW_virtual_word(diff_16);
@@ -272,31 +203,7 @@ void BX_CPU_C::SUB_EwGwM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::SUB_EwGwR(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->rm());
-  op2_16 = BX_READ_16BIT_REG(i->nnn());
-  diff_16 = op1_16 - op2_16;
-  BX_WRITE_16BIT_REG(i->rm(), diff_16);
-
-  SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
-}
-
-void BX_CPU_C::SUB_GwEwM(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-  op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  diff_16 = op1_16 - op2_16;
-  BX_WRITE_16BIT_REG(i->nnn(), diff_16);
-
-  SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
-}
-
-void BX_CPU_C::SUB_GwEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
@@ -308,7 +215,7 @@ void BX_CPU_C::SUB_GwEwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::SUB_AXIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
@@ -320,40 +227,20 @@ void BX_CPU_C::SUB_AXIw(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CMP_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EwGwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
-  op1_16 = read_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   diff_16 = op1_16 - op2_16;
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CMP_EwGwR(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->rm());
-  op2_16 = BX_READ_16BIT_REG(i->nnn());
-  diff_16 = op1_16 - op2_16;
-
-  SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
-}
-
-void BX_CPU_C::CMP_GwEwM(bxInstruction_c *i)
-{
-  Bit16u op1_16, op2_16, diff_16;
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-  op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  diff_16 = op1_16 - op2_16;
-
-  SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
-}
-
-void BX_CPU_C::CMP_GwEwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_GwEwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
@@ -364,7 +251,7 @@ void BX_CPU_C::CMP_GwEwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CMP_AXIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_AXIw(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16, diff_16;
 
@@ -375,13 +262,13 @@ void BX_CPU_C::CMP_AXIw(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CBW(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CBW(bxInstruction_c *i)
 {
   /* CBW: no flags are effected */
   AX = (Bit8s) AL;
 }
 
-void BX_CPU_C::CWD(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CWD(bxInstruction_c *i)
 {
   /* CWD: no flags are affected */
   if (AX & 0x8000) {
@@ -392,7 +279,7 @@ void BX_CPU_C::CWD(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::XADD_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EwGwM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit16u op1_16, op2_16, sum_16;
@@ -403,7 +290,9 @@ void BX_CPU_C::XADD_EwGwM(bxInstruction_c *i)
    * dst  <-- tmp               | op1 = sum
    */
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   sum_16 = op1_16 + op2_16;
   write_RMW_virtual_word(sum_16);
@@ -414,11 +303,11 @@ void BX_CPU_C::XADD_EwGwM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 #else
   BX_INFO(("XADD_EwGw: not supported on < 80486"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::XADD_EwGwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EwGwR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit16u op1_16, op2_16, sum_16;
@@ -443,22 +332,24 @@ void BX_CPU_C::XADD_EwGwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 #else
   BX_INFO(("XADD_EwGw: not supported on < 80486"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::ADD_EwIwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EwIwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), sum_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   sum_16 = op1_16 + op2_16;
   write_RMW_virtual_word(sum_16);
 
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 }
 
-void BX_CPU_C::ADD_EwIwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EwIwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), sum_16;
 
@@ -469,19 +360,21 @@ void BX_CPU_C::ADD_EwIwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_16(op1_16, op2_16, sum_16);
 }
 
-void BX_CPU_C::ADC_EwIwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EwIwM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
   Bit16u op1_16, op2_16 = i->Iw(), sum_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   sum_16 = op1_16 + op2_16 + temp_CF;
   write_RMW_virtual_word(sum_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_LF_INSTR_ADD_ADC16(temp_CF));
 }
 
-void BX_CPU_C::ADC_EwIwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EwIwR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
   Bit16u op1_16, op2_16 = i->Iw(), sum_16;
@@ -490,21 +383,23 @@ void BX_CPU_C::ADC_EwIwR(bxInstruction_c *i)
   sum_16 = op1_16 + op2_16 + temp_CF;
   BX_WRITE_16BIT_REG(i->rm(), sum_16);
 
-  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_INSTR_ADD_ADC16(temp_CF));
+  SET_FLAGS_OSZAPC_16(op1_16, op2_16, sum_16, BX_LF_INSTR_ADD_ADC16(temp_CF));
 }
 
-void BX_CPU_C::SUB_EwIwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EwIwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   diff_16 = op1_16 - op2_16;
   write_RMW_virtual_word(diff_16);
 
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::SUB_EwIwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EwIwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
 
@@ -515,16 +410,18 @@ void BX_CPU_C::SUB_EwIwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CMP_EwIwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EwIwM(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
 
-  op1_16 = read_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_virtual_word(i->seg(), eaddr);
   diff_16 = op1_16 - op2_16;
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::CMP_EwIwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EwIwR(bxInstruction_c *i)
 {
   Bit16u op1_16, op2_16 = i->Iw(), diff_16;
 
@@ -533,72 +430,62 @@ void BX_CPU_C::CMP_EwIwR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_16(op1_16, op2_16, diff_16);
 }
 
-void BX_CPU_C::NEG_EwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EwM(bxInstruction_c *i)
 {
   Bit16u op1_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
-  op1_16 = (Bit16u) -op1_16;
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
+  op1_16 = - (Bit16s)(op1_16);
   write_RMW_virtual_word(op1_16);
 
-  SET_FLAGS_OSZAPC_RESULT_16(op1_16, BX_INSTR_NEG16);
+  SET_FLAGS_OSZAPC_RESULT_16(op1_16, BX_LF_INSTR_NEG16);
 }
 
-void BX_CPU_C::NEG_EwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EwR(bxInstruction_c *i)
 {
   Bit16u op1_16 = BX_READ_16BIT_REG(i->rm());
-  op1_16 = (Bit16u) -op1_16;
+  op1_16 = - (Bit16s)(op1_16);
   BX_WRITE_16BIT_REG(i->rm(), op1_16);
 
-  SET_FLAGS_OSZAPC_RESULT_16(op1_16, BX_INSTR_NEG16);
+  SET_FLAGS_OSZAPC_RESULT_16(op1_16, BX_LF_INSTR_NEG16);
 }
 
-void BX_CPU_C::INC_EwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EwM(bxInstruction_c *i)
 {
   Bit16u op1_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16++;
   write_RMW_virtual_word(op1_16);
 
   SET_FLAGS_OSZAPC_INC_16(op1_16);
 }
 
-void BX_CPU_C::INC_EwR(bxInstruction_c *i)
-{
-  Bit16u op1_16 = BX_READ_16BIT_REG(i->rm());
-  op1_16++;
-  BX_WRITE_16BIT_REG(i->rm(), op1_16);
-
-  SET_FLAGS_OSZAPC_INC_16(op1_16);
-}
-
-void BX_CPU_C::DEC_EwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EwM(bxInstruction_c *i)
 {
   Bit16u op1_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   op1_16--;
   write_RMW_virtual_word(op1_16);
 
   SET_FLAGS_OSZAPC_DEC_16(op1_16);
 }
 
-void BX_CPU_C::DEC_EwR(bxInstruction_c *i)
-{
-  Bit16u op1_16 = BX_READ_16BIT_REG(i->rm());
-  op1_16--;
-  BX_WRITE_16BIT_REG(i->rm(), op1_16);
-
-  SET_FLAGS_OSZAPC_DEC_16(op1_16);
-}
-
-void BX_CPU_C::CMPXCHG_EwGwM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EwGwM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit16u op1_16, op2_16, diff_16;
 
-  op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   diff_16 = AX - op1_16;
   SET_FLAGS_OSZAPC_SUB_16(AX, op1_16, diff_16);
 
@@ -613,11 +500,11 @@ void BX_CPU_C::CMPXCHG_EwGwM(bxInstruction_c *i)
   }
 #else
   BX_INFO(("CMPXCHG_EwGw: not supported for cpu-level <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::CMPXCHG_EwGwR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EwGwR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit16u op1_16, op2_16, diff_16;
@@ -637,6 +524,6 @@ void BX_CPU_C::CMPXCHG_EwGwR(bxInstruction_c *i)
   }
 #else
   BX_INFO(("CMPXCHG_EwGw: not supported for cpu-level <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
