@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer32.cc,v 1.55 2008/03/22 21:29:39 sshwarts Exp $
+// $Id: data_xfer32.cc,v 1.64 2009/04/07 16:12:19 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -52,16 +52,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_ERXId(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->opcodeReg(), i->Id());
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdGdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV32_EdGdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_dword(i->seg(), RMAddr(i), BX_READ_32BIT_REG(i->nnn()));
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdGdR(bxInstruction_c *i)
-{
-  BX_WRITE_32BIT_REGZ(i->rm(), BX_READ_32BIT_REG(i->nnn()));
+  write_virtual_dword_32(i->seg(), eaddr, BX_READ_32BIT_REG(i->nnn()));
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GdEdR(bxInstruction_c *i)
@@ -69,43 +64,43 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GdEdR(bxInstruction_c *i)
   BX_WRITE_32BIT_REGZ(i->nnn(), BX_READ_32BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GdEdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV32_GdEdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit32u val32 = read_virtual_dword(i->seg(), RMAddr(i));
+  Bit32u val32 = read_virtual_dword_32(i->seg(), eaddr);
   BX_WRITE_32BIT_REGZ(i->nnn(), val32);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LEA_GdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  BX_WRITE_32BIT_REGZ(i->nnn(), RMAddr(i));
+  BX_WRITE_32BIT_REGZ(i->nnn(), eaddr);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EAXOd(bxInstruction_c *i)
 {
-  RAX = read_virtual_dword(i->seg(), i->Id());
+  RAX = read_virtual_dword_32(i->seg(), i->Id());
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_OdEAX(bxInstruction_c *i)
 {
-  write_virtual_dword(i->seg(), i->Id(), EAX);
+  write_virtual_dword_32(i->seg(), i->Id(), EAX);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EdIdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_dword(i->seg(), RMAddr(i), i->Id());
+  write_virtual_dword(i->seg(), eaddr, i->Id());
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit8u op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
+  Bit8u op2_8 = read_virtual_byte(i->seg(), eaddr);
 
   /* zero extend byte op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_8);
@@ -121,9 +116,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEbR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEwM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op2_16 = read_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op2_16 = read_virtual_word(i->seg(), eaddr);
 
   /* zero extend word op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit32u) op2_16);
@@ -139,9 +134,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GdEwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit8u op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
+  Bit8u op2_8 = read_virtual_byte(i->seg(), eaddr);
 
   /* sign extend byte op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit8s) op2_8);
@@ -157,9 +152,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEbR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GdEwM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op2_16 = read_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op2_16 = read_virtual_word(i->seg(), eaddr);
 
   /* sign extend word op2 into dword op1 */
   BX_WRITE_32BIT_REGZ(i->nnn(), (Bit16s) op2_16);
@@ -177,9 +172,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EdGdM(bxInstruction_c *i)
 {
   Bit32u op2_32, op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   write_RMW_virtual_dword(op2_32);
 
@@ -199,23 +194,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EdGdR(bxInstruction_c *i)
 //       of whether condition is true or not.  Thus, exceptions may
 //       occur even if the MOV does not take place.
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_OF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVO_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GdEdR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 6
@@ -225,24 +203,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVO_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (!get_OF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNO_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -255,24 +216,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNO_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_CF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVB_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -285,24 +229,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVB_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (!get_CF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNB_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -315,24 +242,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNB_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_ZF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVZ_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -345,24 +255,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVZ_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (!get_ZF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNZ_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -375,24 +268,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNZ_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_CF() || get_ZF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVBE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -405,24 +281,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVBE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (! (get_CF() || get_ZF()))
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNBE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -435,24 +294,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNBE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_SF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVS_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -465,24 +307,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVS_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (!get_SF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNS_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -495,24 +320,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNS_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_PF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVP_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -525,24 +333,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVP_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (!get_PF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNP_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -555,24 +346,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNP_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (getB_SF() != getB_OF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVL_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -585,24 +359,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVL_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (getB_SF() == getB_OF())
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNL_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -615,24 +372,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNL_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (get_ZF() || (getB_SF() != getB_OF()))
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVLE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -645,24 +385,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVLE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
-#endif
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GdEdM(bxInstruction_c *i)
-{
-#if BX_CPU_LEVEL >= 6
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-
-  if (! get_ZF() && (getB_SF() == getB_OF()))
-    BX_WRITE_32BIT_REGZ(i->nnn(), op2_32);
-
-  BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
-#else
-  BX_INFO(("CMOVNLE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -675,6 +398,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GdEdR(bxInstruction_c *i)
   BX_CLEAR_64BIT_HIGH(i->nnn()); // always clear upper part of the register
 #else
   BX_INFO(("CMOVNLE_GdEd: --enable-cpu-level=6 required"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
