@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: icache.h,v 1.48 2009/04/06 18:27:30 sshwarts Exp $
+// $Id: icache.h,v 1.50 2009/10/15 20:50:33 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2007 Stanislav Shwartsman
+//   Copyright (c) 2007-2009 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -90,9 +90,6 @@ public:
       // effectively invalidated.
       pageWriteStampTable[index] = (pageWriteStampTable[index] - 1) & ~ICacheWriteStampFetchModeMask;
     }
-#if BX_DEBUGGER
-    BX_DBG_DIRTY_PAGE(index);
-#endif
   }
 
   BX_CPP_INLINE void resetWriteStamps(void);
