@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical32.cc,v 1.39 2008/03/22 21:29:40 sshwarts Exp $
+// $Id: logical32.cc,v 1.44 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -39,9 +39,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 ^= op2_32;
   write_RMW_virtual_dword(op1_32);
@@ -49,32 +49,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  op1_32 ^= op2_32;
-  BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  op1_32 ^= op2_32;
-  BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
@@ -102,9 +76,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 ^= i->Id();
   write_RMW_virtual_dword(op1_32);
 
@@ -124,9 +98,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 |= i->Id();
   write_RMW_virtual_dword(op1_32);
 
@@ -146,9 +120,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 = ~op1_32;
   write_RMW_virtual_dword(op1_32);
 }
@@ -164,9 +138,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 |= op2_32;
   write_RMW_virtual_dword(op1_32);
@@ -174,32 +148,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  op1_32 |= op2_32;
-  BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  op1_32 |= op2_32;
-  BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
@@ -228,9 +176,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   op1_32 &= op2_32;
   write_RMW_virtual_dword(op1_32);
@@ -238,32 +186,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  op1_32 &= op2_32;
-  BX_WRITE_32BIT_REGZ(i->rm(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  op1_32 &= op2_32;
-  BX_WRITE_32BIT_REGZ(i->nnn(), op1_32);
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
@@ -292,9 +214,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 &= i->Id();
   write_RMW_virtual_dword(op1_32);
 
@@ -310,19 +232,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EdIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdGdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  op1_32 &= op2_32;
-
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdGdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32;
@@ -345,15 +254,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdIdM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit32u op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  op1_32 &= i->Id();
-  SET_FLAGS_OSZAPC_LOGIC_32(op1_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EdIdR(bxInstruction_c *i)
 {
   Bit32u op1_32 = BX_READ_32BIT_REG(i->rm());
