@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical64.cc,v 1.28 2008/05/10 18:10:52 sshwarts Exp $
+// $Id: logical64.cc,v 1.33 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -36,9 +36,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   op1_64 ^= op2_64;
   write_RMW_virtual_qword(op1_64);
@@ -46,34 +46,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  op1_64 ^= op2_64;
-  BX_WRITE_64BIT_REG(i->rm(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  op1_64 ^= op2_64;
-
-  /* now write result back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
@@ -105,9 +77,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64 = (Bit32s) i->Id();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64 ^= op2_64;
   write_RMW_virtual_qword(op1_64);
 
@@ -129,9 +101,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64 = (Bit32s) i->Id();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64 |= op2_64;
   write_RMW_virtual_qword(op1_64);
 
@@ -153,9 +125,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64 = ~op1_64;
   write_RMW_virtual_qword(op1_64);
 }
@@ -171,9 +143,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   op1_64 |= op2_64;
   write_RMW_virtual_qword(op1_64);
@@ -181,34 +153,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  op1_64 |= op2_64;
-  BX_WRITE_64BIT_REG(i->rm(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  op1_64 |= op2_64;
-
-  /* now write result back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
@@ -240,9 +184,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   op1_64 &= op2_64;
   write_RMW_virtual_qword(op1_64);
@@ -250,34 +194,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  op1_64 &= op2_64;
-  BX_WRITE_64BIT_REG(i->rm(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  op1_64 &= op2_64;
-
-  /* now write result back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), op1_64);
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
@@ -308,9 +224,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64 = (Bit32s) i->Id();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64 &= op2_64;
   write_RMW_virtual_qword(op1_64);
 
@@ -328,19 +244,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EqIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqGqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  op1_64 &= op2_64;
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqGqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
@@ -363,19 +266,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_RAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqIdM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  op2_64 = (Bit32s) i->Id();
-  op1_64 &= op2_64;
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EqIdR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64;
