@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logical8.cc,v 1.39 2008/03/22 21:29:40 sshwarts Exp $
+// $Id: logical8.cc,v 1.43 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -34,9 +34,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 ^= op2;
   write_RMW_virtual_byte(op1);
@@ -44,32 +44,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op1 ^= op2;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  op1 ^= op2;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
@@ -97,9 +71,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XOR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1 ^= op2;
   write_RMW_virtual_byte(op1);
 
@@ -121,9 +95,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1 |= op2;
   write_RMW_virtual_byte(op1);
 
@@ -145,9 +119,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::NOT_EbM(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1_8 = ~op1_8;
   write_RMW_virtual_byte(op1_8);
 }
@@ -163,9 +137,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 |= op2;
   write_RMW_virtual_byte(op1);
@@ -173,32 +147,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op1 |= op2;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  op1 |= op2;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::OR_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
@@ -227,9 +175,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   op1 &= op2;
   write_RMW_virtual_byte(op1);
@@ -237,32 +185,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op1 &= op2;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  op1 &= op2;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), op1);
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
@@ -291,9 +213,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1 &= op2;
   write_RMW_virtual_byte(op1);
 
@@ -311,19 +233,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::AND_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1 = read_virtual_byte(i->seg(), RMAddr(i));
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op1 &= op2;
-
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbGbR(bxInstruction_c *i)
 {
   Bit8u op1, op2;
@@ -346,15 +255,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_LOGIC_8(op1);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit8u op1 = read_virtual_byte(i->seg(), RMAddr(i));
-  op1 &= i->Ib();
-  SET_FLAGS_OSZAPC_LOGIC_8(op1);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::TEST_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
