@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: shift16.cc,v 1.48 2009/01/16 18:18:58 sshwarts Exp $
+// $Id: shift16.cc,v 1.51 2009/10/13 15:28:16 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -69,7 +69,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwM(bxInstruction_c *i)
 
   write_RMW_virtual_word(result_16);
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (temp_32 >> (32 - count)) & 0x1;
   of = cf ^ (result_16 >> 15); // of = cf ^ result15
@@ -111,7 +111,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHLD_EwGwR(bxInstruction_c *i)
 
   BX_WRITE_16BIT_REG(i->rm(), result_16);
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (temp_32 >> (32 - count)) & 0x1;
   of = cf ^ (result_16 >> 15); // of = cf ^ result15
@@ -156,10 +156,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwM(bxInstruction_c *i)
 
   write_RMW_virtual_word(result_16);
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = ((result_16 << 1) ^ result_16) >> 15; // of = result14 ^ result15
+  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -197,10 +197,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHRD_EwGwR(bxInstruction_c *i)
 
   BX_WRITE_16BIT_REG(i->rm(), result_16);
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = ((result_16 << 1) ^ result_16) >> 15; // of = result14 ^ result15
+  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result14 ^ result15
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -388,7 +388,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::RCR_Ew(bxInstruction_c *i)
   }
 
   cf = (op1_16 >> (count - 1)) & 0x1;
-  of = ((result_16 << 1) ^ result_16) >> 15; // of = result15 ^ result14
+  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1; // of = result15 ^ result14
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -434,7 +434,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHL_Ew(bxInstruction_c *i)
     write_RMW_virtual_word(result_16);
   }
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -476,9 +476,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SHR_Ew(bxInstruction_c *i)
   cf = (op1_16 >> (count - 1)) & 0x1;
   // note, that of == result15 if count == 1 and
   //            of == 0        if count >= 2
-  of = ((result_16 << 1) ^ result_16) >> 15;
+  of = (((result_16 << 1) ^ result_16) >> 15) & 0x1;
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
   SET_FLAGS_OxxxxC(of, cf);
 }
 
@@ -527,7 +527,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SAR_Ew(bxInstruction_c *i)
     cf = (result_16 & 0x1);
   }
 
-  SET_FLAGS_OSZAPC_LOGIC_16(result_16); /* handle SF, ZF and AF flags */
+  SET_FLAGS_OSZAPC_LOGIC_16(result_16);
   /* signed overflow cannot happen in SAR instruction */
   SET_FLAGS_OxxxxC(0, cf);
 
