@@ -1,8 +1,8 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_move.cc,v 1.98 2009/03/10 21:43:11 sshwarts Exp $
+// $Id: sse_move.cc,v 1.102 2009/10/27 20:03:35 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
-//   Copyright (c) 2003 Stanislav Shwartsman
+//   Copyright (c) 2003-2009 Stanislav Shwartsman
 //          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
 //
 //  This library is free software; you can redistribute it and/or
@@ -358,6 +358,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
 
   BX_CPU_THIS_PTR the_i387.twd = unpack_FPU_TW(tag_byte);
 
+  /* check for unmasked exceptions */
+  if (FPU_PARTIAL_STATUS & ~FPU_CONTROL_WORD & FPU_CW_Exceptions_Mask) {
+    /* set the B and ES bits in the status-word */
+    FPU_PARTIAL_STATUS |= FPU_SW_Summary | FPU_SW_Backward;
+  }
+  else {
+    /* clear the B and ES bits in the status-word */
+    FPU_PARTIAL_STATUS &= ~(FPU_SW_Summary | FPU_SW_Backward);
+  }
+
 #if BX_SUPPORT_X86_64
   if (BX_CPU_THIS_PTR efer.get_FFXSR() && CPL == 0 && Is64BitMode())
     return; // skip restore of the XMM state
@@ -395,22 +405,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXRSTOR(bxInstruction_c *i)
 /* MOVUPS:    0F 10 */
 /* MOVUPD: 66 0F 10 */
 /* MOVDQU: F3 0F 6F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_VpsWps(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_VpsWpsM(bxInstruction_c *i)
 {
 #if BX_SUPPORT_SSE >= 1
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op;
 
-  /* op is a register or memory reference */
-  if (i->modC0()) {
-    op = BX_READ_XMM_REG(i->rm());
-  }
-  else {
-    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    read_virtual_dqword(i->seg(), eaddr, (Bit8u *) &op);
-  }
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  /* pointer, segment address pair */
+  read_virtual_dqword(i->seg(), eaddr, (Bit8u *) &op);
 
   /* now write result back to destination */
   BX_WRITE_XMM_REG(i->nnn(), op);
@@ -423,22 +427,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_VpsWps(bxInstruction_c *i)
 /* MOVUPS:    0F 11 */
 /* MOVUPD: 66 0F 11 */
 /* MOVDQU: F3 0F 7F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_WpsVps(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_WpsVpsM(bxInstruction_c *i)
 {
 #if BX_SUPPORT_SSE >= 1
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->nnn());
 
-  /* op is a register or memory reference */
-  if (i->modC0()) {
-    BX_WRITE_XMM_REG(i->rm(), op);
-  }
-  else {
-    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    write_virtual_dqword(i->seg(), eaddr, (Bit8u *) &op);
-  }
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  /* pointer, segment address pair */
+  write_virtual_dqword(i->seg(), eaddr, (Bit8u *) &op);
 #else
   BX_INFO(("MOVUPS_WpsVps: required SSE, use --enable-sse option"));
   exception(BX_UD_EXCEPTION, 0, 0);
@@ -448,22 +446,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVUPS_WpsVps(bxInstruction_c *i)
 /* MOVAPS:    0F 28 */
 /* MOVAPD: 66 0F 28 */
 /* MOVDQA: F3 0F 6F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWps(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsR(bxInstruction_c *i)
+{
+#if BX_SUPPORT_SSE >= 1
+  BX_CPU_THIS_PTR prepareSSE();
+  BX_WRITE_XMM_REG(i->nnn(), BX_READ_XMM_REG(i->rm()));
+#else
+  BX_INFO(("MOVAPS_VpsWps: required SSE, use --enable-sse option"));
+  exception(BX_UD_EXCEPTION, 0, 0);
+#endif
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWpsM(bxInstruction_c *i)
 {
 #if BX_SUPPORT_SSE >= 1
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op;
 
-  /* op is a register or memory reference */
-  if (i->modC0()) {
-    op = BX_READ_XMM_REG(i->rm());
-  }
-  else {
-    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    read_virtual_dqword_aligned(i->seg(), eaddr, (Bit8u *) &op);
-  }
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  /* pointer, segment address pair */
+  read_virtual_dqword_aligned(i->seg(), eaddr, (Bit8u *) &op);
 
   /* now write result back to destination */
   BX_WRITE_XMM_REG(i->nnn(), op);
@@ -476,22 +479,28 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_VpsWps(bxInstruction_c *i)
 /* MOVAPS:    0F 29 */
 /* MOVAPD: 66 0F 29 */
 /* MOVDQA: F3 0F 7F */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_WpsVps(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_WpsVpsR(bxInstruction_c *i)
+{
+#if BX_SUPPORT_SSE >= 1
+  BX_CPU_THIS_PTR prepareSSE();
+
+  BX_WRITE_XMM_REG(i->rm(), BX_READ_XMM_REG(i->nnn()));
+#else
+  BX_INFO(("MOVAPS_WpsVps: required SSE, use --enable-sse option"));
+  exception(BX_UD_EXCEPTION, 0, 0);
+#endif
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVAPS_WpsVpsM(bxInstruction_c *i)
 {
 #if BX_SUPPORT_SSE >= 1
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op = BX_READ_XMM_REG(i->nnn());
 
-  /* op is a register or memory reference */
-  if (i->modC0()) {
-    BX_WRITE_XMM_REG(i->rm(), op);
-  }
-  else {
-    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    write_virtual_dqword_aligned(i->seg(), eaddr, (Bit8u *) &op);
-  }
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  /* pointer, segment address pair */
+  write_virtual_dqword_aligned(i->seg(), eaddr, (Bit8u *) &op);
 #else
   BX_INFO(("MOVAPS_WpsVps: required SSE, use --enable-sse option"));
   exception(BX_UD_EXCEPTION, 0, 0);
@@ -827,7 +836,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MASKMOVDQU_VdqUdq(bxInstruction_c *i)
   if ((mask.xmm64u(0) | mask.xmm64u(1)) == 0) return;
 
   /* implement as read-modify-write for efficiency */
-  read_virtual_dqword(BX_SEG_REG_DS, rdi, (Bit8u *) &temp);
+  read_virtual_dqword(i->seg(), rdi, (Bit8u *) &temp);
 
   for(unsigned j=0; j<16; j++) {
     if(mask.xmmubyte(j) & 0x80) temp.xmmubyte(j) = op.xmmubyte(j);
