@@ -1,6 +1,23 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dis_tables.h,v 1.35 2009/01/19 19:01:03 sshwarts Exp $
+// $Id: dis_tables.h,v 1.37 2009/10/14 20:45:29 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
+//
+//   Copyright (c) 2005-2009 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 #ifndef _BX_DISASM_TABLES_
 #define _BX_DISASM_TABLES_
@@ -147,6 +164,9 @@
 #define Yd &disassembler::Yd
 #define Yq &disassembler::Yq
 
+#define sYq  &disassembler::sYq
+#define sYdq &disassembler::sYdq
+
 #define Jb &disassembler::Jb
 #define Jw &disassembler::Jw
 #define Jd &disassembler::Jd
