@@ -1,3 +1,26 @@
+/////////////////////////////////////////////////////////////////////////
+// $Id: disasm.h,v 1.50 2009/10/14 20:45:29 sshwarts Exp $
+/////////////////////////////////////////////////////////////////////////
+//
+//   Copyright (c) 2005-2009 Stanislav Shwartsman
+//          Written by Stanislav Shwartsman [sshwarts at sourceforge net]
+//
+//  This library is free software; you can redistribute it and/or
+//  modify it under the terms of the GNU Lesser General Public
+//  License as published by the Free Software Foundation; either
+//  version 2 of the License, or (at your option) any later version.
+//
+//  This library is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+//  Lesser General Public License for more details.
+//
+//  You should have received a copy of the GNU Lesser General Public
+//  License along with this library; if not, write to the Free Software
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+//
+/////////////////////////////////////////////////////////////////////////
+
 #ifndef _BX_DISASM_H_
 #define _BX_DISASM_H_
 
@@ -528,6 +551,11 @@ public:
   void Yd(const x86_insn *insn);
   void Yq(const x86_insn *insn);
 
+  // maskmovdq/maskmovdqu
+  void OP_sY(const x86_insn *insn, unsigned size);
+  void sYq(const x86_insn *insn);
+  void sYdq(const x86_insn *insn);
+
   // jump offset
   void Jb(const x86_insn *insn);
   void Jw(const x86_insn *insn);
