@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: siminterface.cc,v 1.206 2009/03/29 11:13:49 vruppert Exp $
+// $Id: siminterface.cc,v 1.208 2009/11/08 20:47:03 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2009  The Bochs Project
@@ -97,11 +97,11 @@ public:
   virtual int read_rc(const char *path);
   virtual int write_rc(const char *path, int overwrite);
   virtual int get_log_file(char *path, int len);
-  virtual int set_log_file(char *path);
+  virtual int set_log_file(const char *path);
   virtual int get_log_prefix(char *prefix, int len);
-  virtual int set_log_prefix(char *prefix);
+  virtual int set_log_prefix(const char *prefix);
   virtual int get_debugger_log_file(char *path, int len);
-  virtual int set_debugger_log_file(char *path);
+  virtual int set_debugger_log_file(const char *path);
   virtual int get_cdrom_options(int drive, bx_list_c **out, int *device = NULL);
   virtual void set_notify_callback(bxevent_handler func, void *arg);
   virtual void get_notify_callback(bxevent_handler *func, void **arg);
@@ -410,7 +410,7 @@ int bx_real_sim_c::get_log_file(char *path, int len)
   return 0;
 }
 
-int bx_real_sim_c::set_log_file(char *path)
+int bx_real_sim_c::set_log_file(const char *path)
 {
   SIM->get_param_string(BXPN_LOG_FILENAME)->set(path);
   return 0;
@@ -422,7 +422,7 @@ int bx_real_sim_c::get_log_prefix(char *prefix, int len)
   return 0;
 }
 
-int bx_real_sim_c::set_log_prefix(char *prefix)
+int bx_real_sim_c::set_log_prefix(const char *prefix)
 {
   SIM->get_param_string(BXPN_LOG_PREFIX)->set(prefix);
   return 0;
@@ -434,7 +434,7 @@ int bx_real_sim_c::get_debugger_log_file(char *path, int len)
   return 0;
 }
 
-int bx_real_sim_c::set_debugger_log_file(char *path)
+int bx_real_sim_c::set_debugger_log_file(const char *path)
 {
   SIM->get_param_string(BXPN_DEBUGGER_LOG_FILENAME)->set(path);
   return 0;
@@ -1412,11 +1412,11 @@ void bx_param_num_c::set_handler(param_event_handler handler)
   //set (get ());
 }
 
-void bx_param_num_c::set_sr_handlers(void *devptr, param_sr_handler save, param_sr_handler restore)
+void bx_param_num_c::set_sr_handlers(void *devptr, param_save_handler save, param_restore_handler restore)
 {
-  this->sr_devptr = devptr;
-  this->save_handler = save;
-  this->restore_handler = restore;
+  sr_devptr = devptr;
+  save_handler = save;
+  restore_handler = restore;
 }
 
 void bx_param_num_c::set_dependent_list(bx_list_c *l)
@@ -1428,7 +1428,7 @@ void bx_param_num_c::set_dependent_list(bx_list_c *l)
 Bit64s bx_param_num_c::get64()
 {
   if (save_handler) {
-    return (*save_handler)(sr_devptr, this, val.number);
+    return (*save_handler)(sr_devptr, this);
   }
   if (handler) {
     // the handler can decide what value to return and/or do some side effect
