@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cdrom.h,v 1.20 2006/03/07 21:11:16 sshwarts Exp $
+// $Id: cdrom.h,v 1.24 2009/04/21 15:37:17 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 
 // Header file for low-level OS specific CDROM emulation
@@ -30,12 +30,12 @@
 
 class cdrom_interface : public logfunctions {
 public:
-  cdrom_interface(char *dev);
+  cdrom_interface(const char *dev);
   virtual ~cdrom_interface(void);
   void init(void);
 
   // Load CD-ROM. Returns 0 if CD is not ready.
-  bx_bool insert_cdrom(char *dev = NULL);
+  bx_bool insert_cdrom(const char *dev = NULL);
 
   // Logically eject the CD.
   void eject_cdrom();
@@ -47,13 +47,13 @@ public:
   Bit32u capacity();
 
   // Read a single block from the CD. Returns 0 on failure.
-  bx_bool read_block(Bit8u* buf, int lba, int blocksize) BX_CPP_AttrRegparmN(3);
+  bx_bool read_block(Bit8u* buf, Bit32u lba, int blocksize) BX_CPP_AttrRegparmN(3);
 
   // Start (spin up) the CD.
   bx_bool start_cdrom();
 
   // Seek for new block address.
-  void seek(int lba);
+  void seek(Bit32u lba);
 
 private:
   int fd;
