@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: ioapic.cc,v 1.37 2008/04/29 22:14:23 sshwarts Exp $
+// $Id: ioapic.cc,v 1.49 2009/02/22 17:37:09 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -22,29 +22,73 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+/////////////////////////////////////////////////////////////////////////
+
+// Define BX_PLUGGABLE in files that can be compiled into plugins.  For
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
+// is used to know when we are exporting symbols and when we are importing.
+#define BX_PLUGGABLE
 
-#include "bochs.h"
-#include "cpu/apic.h"
 #include "iodev.h"
 
 #if BX_SUPPORT_APIC
 
-class bx_ioapic_c bx_ioapic;
-#define LOG_THIS  bx_ioapic.
+#include "ioapic.h"
+
+#define LOG_THIS theIOAPIC->
+
+bx_ioapic_c *theIOAPIC = NULL;
+
+int libioapic_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+{
+  theIOAPIC = new bx_ioapic_c();
+  bx_devices.pluginIOAPIC = theIOAPIC;
+  BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theIOAPIC, BX_PLUGIN_IOAPIC);
+  return(0); // Success
+}
+
+void libioapic_LTX_plugin_fini(void)
+{
+  delete theIOAPIC;
+}
 
 static bx_bool ioapic_read(bx_phy_address a20addr, unsigned len, void *data, void *param)
 {
-  bx_ioapic.read(a20addr, data, len);
+  if((a20addr & ~0x3) != ((a20addr+len-1) & ~0x3)) {
+    BX_PANIC(("I/O APIC read at address 0x" FMT_PHY_ADDRX " spans 32-bit boundary !", a20addr));
+    return 1;
+  }
+  Bit32u value = theIOAPIC->read_aligned(a20addr & ~0x3);
+  if(len == 4) { // must be 32-bit aligned
+    *((Bit32u *)data) = value;
+    return 1;
+  }
+  // handle partial read, independent of endian-ness
+  value >>= (a20addr&3)*8;
+  if (len == 1)
+    *((Bit8u *) data) = value & 0xff;
+  else if (len == 2)
+    *((Bit16u *)data) = value & 0xffff;
+  else
+    BX_PANIC(("Unsupported I/O APIC read at address 0x" FMT_PHY_ADDRX ", len=%d", a20addr, len));
+
   return 1;
 }
 
 static bx_bool ioapic_write(bx_phy_address a20addr, unsigned len, void *data, void *param)
 {
   if (len != 4) {
-    BX_PANIC (("I/O apic write with len=%ld (should be 4)", len));
+    BX_PANIC(("I/O apic write with len=%d (should be 4)", len));
+    return 1;
   }
-  bx_ioapic.write(a20addr, (Bit32u*) data, len);
+
+  if(a20addr & 0xf) {
+    BX_PANIC(("I/O apic write at unaligned address 0x" FMT_PHY_ADDRX, a20addr));
+    return 1;
+  }
+
+  theIOAPIC->write_aligned(a20addr, *((Bit32u*) data));
   return 1;
 }
 
@@ -68,28 +112,25 @@ void bx_io_redirect_entry_t::register_state(bx_param_c *parent)
   BXRS_HEX_PARAM_SIMPLE(parent, hi);
 }
 
-#define BX_IOAPIC_BASE_ADDR (0xfec00000)
-
-bx_ioapic_c::bx_ioapic_c()
-  : bx_generic_apic_c(BX_IOAPIC_BASE_ADDR)
-{
-  put("IOAP");
-  settype(IOAPICLOG);
-}
-
-bx_ioapic_c::~bx_ioapic_c() {}
-
+#define BX_IOAPIC_BASE_ADDR  (0xfec00000)
 #define BX_IOAPIC_DEFAULT_ID (BX_SMP_PROCESSORS)
 
+bx_ioapic_c::bx_ioapic_c(): base_addr(BX_IOAPIC_BASE_ADDR)
+{
+  set_id(BX_IOAPIC_DEFAULT_ID);
+  put("IOAP");
+}
+
 void bx_ioapic_c::init(void)
 {
-  bx_generic_apic_c::init();
   BX_INFO(("initializing I/O APIC"));
-  base_addr = BX_IOAPIC_BASE_ADDR;
-  set_id(BX_IOAPIC_DEFAULT_ID);
-  ioregsel = 0;
-  DEV_register_memory_handlers(&bx_ioapic,
+  DEV_register_memory_handlers(theIOAPIC,
       ioapic_read, ioapic_write, base_addr, base_addr + 0xfff);
+  reset(BX_RESET_HARDWARE);
+}
+
+void bx_ioapic_c::reset(unsigned type)
+{
   // all interrupts masked
   for (int i=0; i<BX_IOAPIC_NUM_PINS; i++) {
     ioredtbl[i].set_lo_part(0x00010000);
@@ -97,49 +138,53 @@ void bx_ioapic_c::init(void)
   }
   intin = 0;
   irr = 0;
+  ioregsel = 0;
 }
 
-void bx_ioapic_c::read_aligned(bx_phy_address address, Bit32u *data)
+Bit32u bx_ioapic_c::read_aligned(bx_phy_address address)
 {
   BX_DEBUG(("IOAPIC: read aligned addr=%08x", address));
   address &= 0xff;
   if (address == 0x00) {
     // select register
-    *data = ioregsel;
-    return;
+    return ioregsel;
   } else {
     if (address != 0x10)
       BX_PANIC(("IOAPIC: read from unsupported address"));
   }
+
+  Bit32u data = 0;
+
   // only reached when reading data register
   switch (ioregsel) {
   case 0x00:  // APIC ID, note this is 4bits, the upper 4 are reserved
-    *data = ((id & APIC_ID_MASK) << 24);
-    return;
+    data = ((id & APIC_ID_MASK) << 24);
+    break;
   case 0x01:  // version
-    *data = BX_IOAPIC_VERSION_ID;
-    return;
+    data = BX_IOAPIC_VERSION_ID;
+    break;
   case 0x02:
     BX_INFO(("IOAPIC: arbitration ID unsupported, returned 0"));
-    *data = 0;
-    return;
+    break;
   default:
     int index = (ioregsel - 0x10) >> 1;
     if (index >= 0 && index < BX_IOAPIC_NUM_PINS) {
       bx_io_redirect_entry_t *entry = ioredtbl + index;
-      *data = (ioregsel&1) ? entry->get_hi_part() : entry->get_lo_part();
-      return;
+      data = (ioregsel&1) ? entry->get_hi_part() : entry->get_lo_part();
+      break;
     }
     BX_PANIC(("IOAPIC: IOREGSEL points to undefined register %02x", ioregsel));
   }
+
+  return data;
 }
 
-void bx_ioapic_c::write_aligned(bx_phy_address address, Bit32u *value)
+void bx_ioapic_c::write_aligned(bx_phy_address address, Bit32u value)
 {
-  BX_DEBUG(("IOAPIC: write aligned addr=%08x, data=%08x", address, *value));
+  BX_DEBUG(("IOAPIC: write aligned addr=%08x, data=%08x", (unsigned) address, value));
   address &= 0xff;
   if (address == 0x00)  {
-    ioregsel = *value;
+    ioregsel = value;
     return;
   } else {
     if (address != 0x10)
@@ -149,7 +194,7 @@ void bx_ioapic_c::write_aligned(bx_phy_address address, Bit32u *value)
   switch (ioregsel) {
     case 0x00: // set APIC ID
       {
-	Bit8u newid = (*value >> 24) & APIC_ID_MASK;
+	Bit8u newid = (value >> 24) & APIC_ID_MASK;
 	BX_INFO(("IOAPIC: setting id to 0x%x", newid));
 	set_id (newid);
 	return;
@@ -163,9 +208,9 @@ void bx_ioapic_c::write_aligned(bx_phy_address address, Bit32u *value)
       if (index >= 0 && index < BX_IOAPIC_NUM_PINS) {
 	bx_io_redirect_entry_t *entry = ioredtbl + index;
 	if (ioregsel&1)
-	  entry->set_hi_part(*value);
+	  entry->set_hi_part(value);
 	else
-	  entry->set_lo_part(*value);
+	  entry->set_lo_part(value);
 	char buf[1024];
 	entry->sprintf_self(buf);
 	BX_DEBUG(("IOAPIC: now entry[%d] is %s", index, buf));
@@ -252,6 +297,7 @@ void bx_ioapic_c::service_ioapic()
 void bx_ioapic_c::register_state(void)
 {
   bx_list_c *list = new bx_list_c(SIM->get_bochs_root(), "ioapic", "IOAPIC State", 4);
+
   BXRS_HEX_PARAM_SIMPLE(list, ioregsel);
   BXRS_HEX_PARAM_SIMPLE(list, intin);
   BXRS_HEX_PARAM_SIMPLE(list, irr);
