@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: pic.cc,v 1.47 2007/11/30 08:56:34 sshwarts Exp $
+// $Id: pic.cc,v 1.55 2009/04/21 20:27:35 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -22,15 +22,16 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
-
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+/////////////////////////////////////////////////////////////////////////
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
-// platforms that require a special tag on exported symbols, BX_PLUGGABLE 
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
 
 #include "iodev.h"
+#include "pic.h"
 
 #define LOG_THIS thePic->
 
@@ -52,7 +53,6 @@ void libpic_LTX_plugin_fini(void)
 bx_pic_c::bx_pic_c(void)
 {
   put("PIC");
-  settype(PICLOG);
 }
 
 bx_pic_c::~bx_pic_c(void)
@@ -270,7 +270,7 @@ void bx_pic_c::write(Bit32u address, Bit32u value, unsigned io_len)
     case 0x20:
       if (value & 0x10) { /* initialization command 1 */
         BX_DEBUG(("master: init command 1 found"));
-        BX_DEBUG(("        requires 4 = %u", (unsigned) (value & 0x01) ));
+        BX_DEBUG(("        requires 4 = %u", (unsigned) (value & 0x01)));
         BX_DEBUG(("        cascade mode: [0=cascade,1=single] %u",
                   (unsigned) ((value & 0x02) >> 1)));
         BX_PIC_THIS s.master_pic.init.in_init = 1;
@@ -295,7 +295,7 @@ void bx_pic_c::write(Bit32u address, Bit32u value, unsigned io_len)
         return;
       }
 
-      if ( (value & 0x18) == 0x08 ) { /* OCW3 */
+      if ((value & 0x18) == 0x08) { /* OCW3 */
         Bit8u special_mask, poll, read_op;
 
         special_mask = (value & 0x60) >> 5;
@@ -445,7 +445,7 @@ void bx_pic_c::write(Bit32u address, Bit32u value, unsigned io_len)
       if (value & 0x10) { /* initialization command 1 */
         BX_DEBUG(("slave: init command 1 found"));
         BX_DEBUG(("       requires 4 = %u",
-            (unsigned) (value & 0x01) ));
+            (unsigned) (value & 0x01)));
         BX_DEBUG(("       cascade mode: [0=cascade,1=single] %u",
             (unsigned) ((value & 0x02) >> 1)));
         BX_PIC_THIS s.slave_pic.init.in_init = 1;
@@ -470,7 +470,7 @@ void bx_pic_c::write(Bit32u address, Bit32u value, unsigned io_len)
         return;
       }
 
-      if ( (value & 0x18) == 0x08 ) { /* OCW3 */
+      if ((value & 0x18) == 0x08) { /* OCW3 */
         Bit8u special_mask, poll, read_op;
 
         special_mask = (value & 0x60) >> 5;
@@ -622,7 +622,7 @@ void bx_pic_c::lower_irq(unsigned irq_no)
 #if BX_SUPPORT_APIC
   // forward this function call to the ioapic too
   if (DEV_ioapic_present() && (irq_no != 2)) {
-    bx_devices.ioapic->set_irq_level(irq_no, 0);
+    DEV_ioapic_set_irq_level(irq_no, 0);
   }
 #endif
 
@@ -644,7 +644,7 @@ void bx_pic_c::raise_irq(unsigned irq_no)
 #if BX_SUPPORT_APIC
   // forward this function call to the ioapic too
   if (DEV_ioapic_present() && (irq_no != 2)) {
-    bx_devices.ioapic->set_irq_level(irq_no, 1);
+    DEV_ioapic_set_irq_level(irq_no, 1);
   }
 #endif
 
@@ -721,13 +721,13 @@ void bx_pic_c::service_master_pic(void)
     isr = BX_PIC_THIS s.master_pic.isr;
     if (isr) {
       max_irq = highest_priority;
-      while ( (isr & (1 << max_irq)) == 0) {
+      while ((isr & (1 << max_irq)) == 0) {
         max_irq++;
         if(max_irq > 7)
           max_irq = 0;
       }
-      if (max_irq == highest_priority ) return; /* Highest priority interrupt in-service,
-                                                 * no other priorities allowed */
+      if (max_irq == highest_priority) return; /* Highest priority interrupt in-service,
+                                                * no other priorities allowed */
       if (max_irq > 7) BX_PANIC(("error in service_master_pic()"));
     }
     else
@@ -735,13 +735,13 @@ void bx_pic_c::service_master_pic(void)
   }
 
   /* now, see if there are any higher priority requests */
-  if ((unmasked_requests = (BX_PIC_THIS s.master_pic.irr & ~BX_PIC_THIS s.master_pic.imr)) ) {
+  if ((unmasked_requests = (BX_PIC_THIS s.master_pic.irr & ~BX_PIC_THIS s.master_pic.imr))) {
     irq = highest_priority;
     do {
       /* for special mode, since we're looking at all IRQ's, skip if
        * current IRQ is already in-service
        */
-      if ( ! (BX_PIC_THIS s.master_pic.special_mask && ((BX_PIC_THIS s.master_pic.isr >> irq) & 0x01)) ) {
+      if (! (BX_PIC_THIS s.master_pic.special_mask && ((BX_PIC_THIS s.master_pic.isr >> irq) & 0x01))) {
         if (unmasked_requests & (1 << irq)) {
           BX_DEBUG(("signalling IRQ(%u)", (unsigned) irq));
           BX_PIC_THIS s.master_pic.INT = 1;
@@ -749,7 +749,7 @@ void bx_pic_c::service_master_pic(void)
           BX_SET_INTR(1);
           return;
         } /* if (unmasked_requests & ... */
-      } 
+      }
 
       irq ++;
       if(irq > 7)
@@ -782,13 +782,13 @@ void bx_pic_c::service_slave_pic(void)
     isr = BX_PIC_THIS s.slave_pic.isr;
     if (isr) {
       max_irq = highest_priority;
-      while ( (isr & (1 << max_irq)) == 0) {
+      while ((isr & (1 << max_irq)) == 0) {
         max_irq++;
         if(max_irq > 7)
           max_irq = 0;
       }
-      if (max_irq == highest_priority ) return; /* Highest priority interrupt in-service,
-                                                 * no other priorities allowed */
+      if (max_irq == highest_priority) return; /* Highest priority interrupt in-service,
+                                                * no other priorities allowed */
       if (max_irq > 7) BX_PANIC(("error in service_master_pic()"));
     }
     else
@@ -796,13 +796,13 @@ void bx_pic_c::service_slave_pic(void)
   }
 
   /* now, see if there are any higher priority requests */
-  if ((unmasked_requests = (BX_PIC_THIS s.slave_pic.irr & ~BX_PIC_THIS s.slave_pic.imr)) ) {
+  if ((unmasked_requests = (BX_PIC_THIS s.slave_pic.irr & ~BX_PIC_THIS s.slave_pic.imr))) {
     irq = highest_priority;
     do {
       /* for special mode, since we're looking at all IRQ's, skip if
        * current IRQ is already in-service
        */
-      if ( ! (BX_PIC_THIS s.slave_pic.special_mask && ((BX_PIC_THIS s.slave_pic.isr >> irq) & 0x01)) ) {
+      if (! (BX_PIC_THIS s.slave_pic.special_mask && ((BX_PIC_THIS s.slave_pic.isr >> irq) & 0x01))) {
         if (unmasked_requests & (1 << irq)) {
           BX_DEBUG(("slave: signalling IRQ(%u)", (unsigned) 8 + irq));
 
@@ -872,16 +872,16 @@ Bit8u bx_pic_c::IAC(void)
   return(vector);
 }
 
-void bx_pic_c::show_pic_state(void)
+#if BX_DEBUGGER
+void bx_pic_c::debug_dump(void)
 {
-#if defined(BX_DEBUGGER) && (BX_DEBUGGER == 1)
-dbg_printf("s.master_pic.imr = %02x\n", BX_PIC_THIS s.master_pic.imr);
-dbg_printf("s.master_pic.isr = %02x\n", BX_PIC_THIS s.master_pic.isr);
-dbg_printf("s.master_pic.irr = %02x\n", BX_PIC_THIS s.master_pic.irr);
-dbg_printf("s.master_pic.irq = %02x\n", BX_PIC_THIS s.master_pic.irq);
-dbg_printf("s.slave_pic.imr = %02x\n", BX_PIC_THIS s.slave_pic.imr);
-dbg_printf("s.slave_pic.isr = %02x\n", BX_PIC_THIS s.slave_pic.isr);
-dbg_printf("s.slave_pic.irr = %02x\n", BX_PIC_THIS s.slave_pic.irr);
-dbg_printf("s.slave_pic.irq = %02x\n", BX_PIC_THIS s.slave_pic.irq);
-#endif
+  dbg_printf("s.master_pic.imr = %02x\n", BX_PIC_THIS s.master_pic.imr);
+  dbg_printf("s.master_pic.isr = %02x\n", BX_PIC_THIS s.master_pic.isr);
+  dbg_printf("s.master_pic.irr = %02x\n", BX_PIC_THIS s.master_pic.irr);
+  dbg_printf("s.master_pic.irq = %02x\n", BX_PIC_THIS s.master_pic.irq);
+  dbg_printf("s.slave_pic.imr = %02x\n", BX_PIC_THIS s.slave_pic.imr);
+  dbg_printf("s.slave_pic.isr = %02x\n", BX_PIC_THIS s.slave_pic.isr);
+  dbg_printf("s.slave_pic.irr = %02x\n", BX_PIC_THIS s.slave_pic.irr);
+  dbg_printf("s.slave_pic.irq = %02x\n", BX_PIC_THIS s.slave_pic.irq);
 }
+#endif
