@@ -1,5 +1,5 @@
 ///////////////////////////////////////////////////////////////////////
-// $Id: pit_wrap.cc,v 1.69 2008/02/15 22:05:43 sshwarts Exp $
+// $Id: pit_wrap.cc,v 1.76 2009/04/11 07:04:52 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -22,13 +22,36 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
+// Define BX_PLUGGABLE in files that can be compiled into plugins.  For
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
+// is used to know when we are exporting symbols and when we are importing.
+#define BX_PLUGGABLE
 
 #include "iodev.h"
-
+#include "pit_wrap.h"
+#include "virt_timer.h"
 #include "speaker.h"
 
+
+#define LOG_THIS thePit->
+
+bx_pit_c *thePit = NULL;
+
+int libpit_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
+{
+  thePit = new bx_pit_c();
+  bx_devices.pluginPitDevice = thePit;
+  BX_REGISTER_DEVICE_DEVMODEL(plugin, type, thePit, BX_PLUGIN_PIT);
+  return(0); // Success
+}
+
+void libpit_LTX_plugin_fini(void)
+{
+  delete thePit;
+}
+
 //Important constant #defines:
 #define USEC_PER_SECOND (1000000)
 //1.193181MHz Clock
@@ -42,22 +65,6 @@
 #define F2I(x)  ((Bit64u)(Bit64s) (x))
 #define I2F(x)  ((double)(Bit64s) (x))
 
-//DEBUG configuration:
-
-//Set up Logging.
-#define LOG_THIS bx_pit.
-
-//A single instance.
-bx_pit_c bx_pit;
-#if BX_USE_PIT_SMF
-#define this (&bx_pit)
-#endif
-
-//Workaround for environments where OUT is defined.
-#ifdef OUT
-#  undef OUT
-#endif
-
 
 //Generic MAX and MIN Functions
 #define BX_MAX(a,b) ( ((a)>(b))?(a):(b) )
@@ -81,17 +88,16 @@ bx_pit_c bx_pit;
 bx_pit_c::bx_pit_c()
 {
   put("PIT");
-  settype(PITLOG);
   s.speaker_data_on=0;
 
   /* 8254 PIT (Programmable Interval Timer) */
 
-  BX_PIT_THIS s.timer_handle[1] = BX_NULL_TIMER_HANDLE;
-  BX_PIT_THIS s.timer_handle[2] = BX_NULL_TIMER_HANDLE;
-  BX_PIT_THIS s.timer_handle[0] = BX_NULL_TIMER_HANDLE;
+  s.timer_handle[1] = BX_NULL_TIMER_HANDLE;
+  s.timer_handle[2] = BX_NULL_TIMER_HANDLE;
+  s.timer_handle[0] = BX_NULL_TIMER_HANDLE;
 }
 
-int bx_pit_c::init(void)
+void bx_pit_c::init(void)
 {
   DEV_register_irq(0, "8254 PIT");
   DEV_register_ioread_handler(this, read_handler, 0x0040, "8254 PIT", 1);
@@ -106,7 +112,7 @@ int bx_pit_c::init(void)
   DEV_register_iowrite_handler(this, write_handler, 0x0043, "8254 PIT", 1);
   DEV_register_iowrite_handler(this, write_handler, 0x0061, "8254 PIT", 1);
 
-  BX_DEBUG(("pit: starting init"));
+  BX_DEBUG(("starting init"));
 
   BX_PIT_THIS s.speaker_data_on = 0;
   BX_PIT_THIS s.refresh_clock_div2 = 0;
@@ -119,13 +125,13 @@ int bx_pit_c::init(void)
   if (BX_PIT_THIS s.timer_handle[0] == BX_NULL_TIMER_HANDLE) {
     BX_PIT_THIS s.timer_handle[0] = bx_virt_timer.register_timer(this, timer_handler, (unsigned) 100 , 1, 1, "pit_wrap");
   }
-  BX_DEBUG(("pit: RESETting timer."));
+  BX_DEBUG(("RESETting timer."));
   bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
   BX_DEBUG(("deactivated timer."));
   if (BX_PIT_THIS s.timer.get_next_event_time()) {
     bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
-				(Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
-				0);
+                                 (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
+                                 0);
     BX_DEBUG(("activated timer."));
   }
   BX_PIT_THIS s.last_next_event_time = BX_PIT_THIS s.timer.get_next_event_time();
@@ -134,20 +140,12 @@ int bx_pit_c::init(void)
   BX_PIT_THIS s.total_ticks=0;
   BX_PIT_THIS s.total_usec=0;
 
-  BX_DEBUG(("pit: finished init"));
+  BX_DEBUG(("finished init"));
 
   BX_DEBUG(("s.last_usec="FMT_LL"d",BX_PIT_THIS s.last_usec));
   BX_DEBUG(("s.timer_id=%d",BX_PIT_THIS s.timer_handle[0]));
   BX_DEBUG(("s.timer.get_next_event_time=%d",BX_PIT_THIS s.timer.get_next_event_time()));
   BX_DEBUG(("s.last_next_event_time=%d",BX_PIT_THIS s.last_next_event_time));
-
-  return(1);
-}
-
-void bx_pit_c::exit(void)
-{
-  BX_PIT_THIS s.timer_handle[0] = BX_NULL_TIMER_HANDLE;
-  BX_PIT_THIS s.timer.init();
 }
 
 void bx_pit_c::reset(unsigned type)
@@ -180,7 +178,7 @@ void bx_pit_c::handle_timer()
   Bit64u time_passed = my_time_usec-BX_PIT_THIS s.last_usec;
   Bit32u time_passed32 = (Bit32u)time_passed;
 
-  BX_DEBUG(("pit: entering timer handler"));
+  BX_DEBUG(("entering timer handler"));
 
   if(time_passed32) {
     periodic(time_passed32);
@@ -188,13 +186,13 @@ void bx_pit_c::handle_timer()
   BX_PIT_THIS s.last_usec=BX_PIT_THIS s.last_usec + time_passed;
   if(time_passed || (BX_PIT_THIS s.last_next_event_time != BX_PIT_THIS s.timer.get_next_event_time()))
   {
-    BX_DEBUG(("pit: RESETting timer"));
+    BX_DEBUG(("RESETting timer"));
     bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
     BX_DEBUG(("deactivated timer"));
     if(BX_PIT_THIS s.timer.get_next_event_time()) {
       bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
-				  (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
-				  0);
+                                   (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
+                                   0);
       BX_DEBUG(("activated timer"));
     }
     BX_PIT_THIS s.last_next_event_time = BX_PIT_THIS s.timer.get_next_event_time();
@@ -220,43 +218,43 @@ Bit32u bx_pit_c::read(Bit32u address, unsigned io_len)
 #else
   UNUSED(this_ptr);
 #endif  // !BX_USE_PIT_SMF
-  BX_DEBUG(("pit: entering read handler"));
+
+  Bit8u value = 0;
 
   handle_timer();
 
   Bit64u my_time_usec = bx_virt_timer.time_usec();
 
-  if (bx_dbg.pit)
-    BX_INFO(("pit: io read from port %04x", (unsigned) address));
-
   switch (address) {
 
     case 0x40: /* timer 0 - system ticks */
-      return(BX_PIT_THIS s.timer.read(0));
+      value = BX_PIT_THIS s.timer.read(0);
       break;
     case 0x41: /* timer 1 read */
-      return(BX_PIT_THIS s.timer.read(1));
+      value = BX_PIT_THIS s.timer.read(1);
       break;
     case 0x42: /* timer 2 read */
-      return(BX_PIT_THIS s.timer.read(2));
+      value = BX_PIT_THIS s.timer.read(2);
       break;
     case 0x43: /* timer 1 read */
-      return(BX_PIT_THIS s.timer.read(3));
+      value = BX_PIT_THIS s.timer.read(3);
       break;
 
     case 0x61:
       /* AT, port 61h */
       BX_PIT_THIS s.refresh_clock_div2 = (bx_bool)((my_time_usec / 15) & 1);
-      return (BX_PIT_THIS s.timer.read_OUT(2)<<5) |
-             (BX_PIT_THIS s.refresh_clock_div2<<4) |
-             (BX_PIT_THIS s.speaker_data_on<<1) |
-             (BX_PIT_THIS s.timer.read_GATE(2)?1:0);
+      value = (BX_PIT_THIS s.timer.read_OUT(2)<<5) |
+              (BX_PIT_THIS s.refresh_clock_div2<<4) |
+              (BX_PIT_THIS s.speaker_data_on<<1) |
+              (BX_PIT_THIS s.timer.read_GATE(2)?1:0);
       break;
 
     default:
-      BX_PANIC(("pit: unsupported io read from port %04x", address));
+      BX_PANIC(("unsupported io read from port 0x%04x", address));
   }
-  return(0); /* keep compiler happy */
+
+  BX_DEBUG(("read from port 0x%04x, value = 0x%02x", address, value));
+  return value;
 }
 
 // static IO port write callback handler
@@ -279,8 +277,6 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
   Bit64u time_passed = my_time_usec-BX_PIT_THIS s.last_usec;
   Bit32u time_passed32 = (Bit32u)time_passed;
 
-  BX_DEBUG(("pit: entering write handler"));
-
   if(time_passed32) {
     periodic(time_passed32);
   }
@@ -288,9 +284,7 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
 
   value = (Bit8u) dvalue;
 
-  if (bx_dbg.pit)
-    BX_INFO(("pit: write to port %04x = %02x",
-      (unsigned) address, (unsigned) value));
+  BX_DEBUG(("write to port 0x%04x, value = 0x%02x", address, value));
 
   switch (address) {
     case 0x40: /* timer 0: write count register */
@@ -312,28 +306,27 @@ void bx_pit_c::write(Bit32u address, Bit32u dvalue, unsigned io_len)
     case 0x61:
       BX_PIT_THIS s.speaker_data_on = (value >> 1) & 0x01;
       if (BX_PIT_THIS s.speaker_data_on) {
-	  DEV_speaker_beep_on((float)(1193180.0 / this->get_timer(2)));
+        DEV_speaker_beep_on((float)(1193180.0 / BX_PIT_THIS get_timer(2)));
       } else {
-	  DEV_speaker_beep_off();
+        DEV_speaker_beep_off();
       }
       /* ??? only on AT+ */
       BX_PIT_THIS s.timer.set_GATE(2, value & 0x01);
       break;
 
     default:
-      BX_PANIC(("pit: unsupported io write to port %04x = %02x",
-        (unsigned) address, (unsigned) value));
+      BX_PANIC(("unsupported io write to port 0x%04x = 0x%02x", address, value));
   }
 
   if(time_passed || (BX_PIT_THIS s.last_next_event_time != BX_PIT_THIS s.timer.get_next_event_time()))
   {
-    BX_DEBUG(("pit: RESETting timer"));
+    BX_DEBUG(("RESETting timer"));
     bx_virt_timer.deactivate_timer(BX_PIT_THIS s.timer_handle[0]);
     BX_DEBUG(("deactivated timer"));
     if(BX_PIT_THIS s.timer.get_next_event_time()) {
       bx_virt_timer.activate_timer(BX_PIT_THIS s.timer_handle[0],
-				  (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
-				  0);
+                                   (Bit32u)BX_MAX(1,TICKS_TO_USEC(BX_PIT_THIS s.timer.get_next_event_time())),
+                                   0);
       BX_DEBUG(("activated timer"));
     }
     BX_PIT_THIS s.last_next_event_time = BX_PIT_THIS s.timer.get_next_event_time();
@@ -379,3 +372,7 @@ void bx_pit_c::irq_handler(bx_bool value)
     DEV_pic_lower_irq(0);
   }
 }
+
+Bit16u bx_pit_c::get_timer(int Timer) {
+  return BX_PIT_THIS s.timer.get_inlatch(Timer);
+}
