@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: speaker.cc,v 1.10 2006/09/10 17:18:44 vruppert Exp $
+// $Id: speaker.cc,v 1.15 2009/04/10 08:15:25 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright 2003 by David N. Welton <davidw@dedasys.com>.
@@ -16,7 +16,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 #define BX_PLUGGABLE
 
@@ -54,8 +54,7 @@ void libspeaker_LTX_plugin_fini(void)
 
 bx_speaker_c::bx_speaker_c()
 {
-  put("SPEAKER");
-  settype(SPEAKERLOG);
+  put("SPEAK");
 
   beep_frequency = 0.0; // Off
 
@@ -75,7 +74,7 @@ bx_speaker_c::~bx_speaker_c()
   BX_DEBUG(("Exit"));
 }
 
-void bx_speaker_c::init(void) 
+void bx_speaker_c::init(void)
 {
 #ifdef __linux__
   if (consolefd != -1) {
@@ -89,7 +88,7 @@ void bx_speaker_c::init(void)
   this->beep_off();
 }
 
-void bx_speaker_c::reset(unsigned type) 
+void bx_speaker_c::reset(unsigned type)
 {
   beep_off();
 }
