@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: unmapped.cc,v 1.27 2008/02/15 22:05:43 sshwarts Exp $
+// $Id: unmapped.cc,v 1.36 2009/04/26 06:56:28 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
@@ -31,6 +31,7 @@
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
 #include "iodev.h"
+#include "unmapped.h"
 
 #define LOG_THIS theUnmappedDevice->
 
@@ -39,7 +40,6 @@ bx_unmapped_c *theUnmappedDevice = NULL;
 int libunmapped_LTX_plugin_init(plugin_t *plugin, plugintype_t type, int argc, char *argv[])
 {
   theUnmappedDevice = new bx_unmapped_c();
-  bx_devices.pluginUnmapped = theUnmappedDevice;
   BX_REGISTER_DEVICE_DEVMODEL(plugin, type, theUnmappedDevice, BX_PLUGIN_UNMAPPED);
   return(0); // Success
 }
@@ -52,7 +52,6 @@ void libunmapped_LTX_plugin_fini(void)
 bx_unmapped_c::bx_unmapped_c(void)
 {
   put("UNMP");
-  settype(UNMAPLOG);
 }
 
 bx_unmapped_c::~bx_unmapped_c(void)
@@ -68,6 +67,7 @@ void bx_unmapped_c::init(void)
   s.port80 = 0x00;
   s.port8e = 0x00;
   s.shutdown = 0;
+  s.port_e9_hack = SIM->get_param_bool(BXPN_PORT_E9_HACK)->get();
 }
 
 // static IO port read callback handler
@@ -103,16 +103,21 @@ Bit32u bx_unmapped_c::read(Bit32u address, unsigned io_len)
     case 0x8e:
       retval = BX_UM_THIS s.port8e;
       break;
-#if BX_PORT_E9_HACK
+
     // Unused port on ISA - this can be used by the emulated code
     // to detect it is running inside Bochs and that the debugging
     // features are available (write 0xFF or something on unused
     // port 0x80, then read from 0xe9, if value is 0xe9, debug
     // output is available) (see write() for that) -- Andreas and Emmanuel
     case 0xe9:
-      retval = 0xe9;
+      if (BX_UM_THIS s.port_e9_hack) {
+         retval = 0xe9;
+      }
+      else {
+         retval = 0xffffffff;
+      }
       break;
-#endif
+
     case 0x03df:
       retval = 0xffffffff;
       BX_DEBUG(("unsupported IO read from port %04x (CGA)", address));
@@ -138,7 +143,6 @@ Bit32u bx_unmapped_c::read(Bit32u address, unsigned io_len)
     case 0x03fd: /* UART */
     case 0x17c6:
       retval = 0xffffffff;
-      BX_DEBUG(("unsupported IO read from port %04x", address));
       break;
     default:
       retval = 0xffffffff;
@@ -146,8 +150,7 @@ Bit32u bx_unmapped_c::read(Bit32u address, unsigned io_len)
 
 return_from_read:
 
-  if (bx_dbg.unsupported_io) {
-    switch (io_len) {
+  switch (io_len) {
     case 1:
       retval = (Bit8u)retval;
       BX_DEBUG(("unmapped: 8-bit read from %04x = %02x", address, retval));
@@ -160,8 +163,7 @@ return_from_read:
       BX_DEBUG(("unmapped: 32-bit read from %04x = %08x", address, retval));
       break;
     default:
-      BX_DEBUG(("unmapped: %d-bit read from %04x = %x", io_len * 8, address, retval));
-    }
+      BX_PANIC(("unmapped: %d-bit read from %04x = %x", io_len * 8, address, retval));
   }
 
   return retval;
@@ -201,7 +203,6 @@ void bx_unmapped_c::write(Bit32u address, Bit32u value, unsigned io_len)
       BX_UM_THIS s.port8e = value;
       break;
 
-#if BX_PORT_E9_HACK
     // This port doesn't exist on normal ISA architecture. However,
     // we define a convention here, to display on the console of the
     // system running Bochs, anything that is written to it. The
@@ -212,10 +213,11 @@ void bx_unmapped_c::write(Bit32u address, Bit32u value, unsigned io_len)
     // Idea by Andreas Beck (andreas.beck@ggi-project.org)
 
     case 0xe9:
-      putchar(value);
-      fflush(stdout);
+      if (BX_UM_THIS s.port_e9_hack) {
+        putchar(value);
+        fflush(stdout);
+      }
       break;
-#endif
 
     case 0xed: // Dummy port used as I/O delay
       break;
@@ -249,10 +251,10 @@ void bx_unmapped_c::write(Bit32u address, Bit32u value, unsigned io_len)
         case 'n': if (BX_UM_THIS s.shutdown == 7) BX_UM_THIS s.shutdown = 8; break;
 #if BX_DEBUGGER
         // Very handy for debugging:
-	// output 'D' to port 8900, and bochs quits to debugger
+        // output 'D' to port 8900, and bochs quits to debugger
         case 'D': bx_debug_break(); break;
 #endif
-	default : BX_UM_THIS s.shutdown = 0; break;
+        default : BX_UM_THIS s.shutdown = 0; break;
       }
       if (BX_UM_THIS s.shutdown == 8) {
         bx_user_quit = 1;
@@ -272,20 +274,18 @@ void bx_unmapped_c::write(Bit32u address, Bit32u value, unsigned io_len)
 
 return_from_write:
 
-  if (bx_dbg.unsupported_io) {
-    switch (io_len) {
-      case 1:
-        BX_INFO(("unmapped: 8-bit write to %04x = %02x", address, value));
-        break;
-      case 2:
-        BX_INFO(("unmapped: 16-bit write to %04x = %04x", address, value));
-        break;
-      case 4:
-        BX_INFO(("unmapped: 32-bit write to %04x = %08x", address, value));
-        break;
-      default:
-        BX_INFO(("unmapped: %d-bit write to %04x = %x", io_len * 8, address, value));
-        break;
-    }
+  switch (io_len) {
+    case 1:
+      BX_DEBUG(("unmapped: 8-bit write to %04x = %02x", address, value));
+      break;
+    case 2:
+      BX_DEBUG(("unmapped: 16-bit write to %04x = %04x", address, value));
+      break;
+    case 4:
+      BX_DEBUG(("unmapped: 32-bit write to %04x = %08x", address, value));
+      break;
+    default:
+      BX_PANIC(("unmapped: %d-bit write to %04x = %x", io_len * 8, address, value));
+      break;
   }
 }
