@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: virt_timer.h,v 1.15 2008/02/15 22:05:43 sshwarts Exp $
+// $Id: virt_timer.h,v 1.19 2009/02/08 09:05:52 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #ifndef _BX_VIRT_TIMER_H
@@ -34,7 +34,7 @@
 
 #define BX_MAX_VIRTUAL_TIME (0x7fffffff)
 
-class bx_virt_timer_c : public logfunctions {
+class BOCHSAPI bx_virt_timer_c : public logfunctions {
 private:
 
   struct {
@@ -79,6 +79,9 @@ private:
   Bit64u last_realtime_ticks;
   Bit64u ticks_per_second;
 
+  // Local copy of IPS value
+  Bit64u ips;
+
   bx_bool init_done;
 
   int system_timer_id;
@@ -152,6 +155,6 @@ public:
   void register_state(void);
 };
 
-extern bx_virt_timer_c bx_virt_timer;
+BOCHSAPI extern bx_virt_timer_c bx_virt_timer;
 
 #endif // _BX_VIRT_TIMER_H
